@@ -11,8 +11,7 @@ const Me = ExtensionUtils.getCurrentExtension();
 const { WindowPicker, WindowPickerToggle } = Me.imports.windowPicker;
 const { WorkspaceIndicator } = Me.imports.workspaceIndicator;
 
-const Gettext = imports.gettext.domain('gnome-shell-extensions');
-const _ = Gettext.gettext;
+const _ = ExtensionUtils.gettext;
 
 const ICON_TEXTURE_SIZE = 24;
 const DND_ACTIVATE_TIMEOUT = 500;
@@ -23,34 +22,10 @@ const GroupingMode = {
     ALWAYS: 2,
 };
 
-
-function _minimizeOrActivateWindow(window) {
-    let focusWindow = global.display.focus_window;
-    if (focusWindow === window ||
-        focusWindow && focusWindow.get_transient_for() === window)
-        window.minimize();
-    else
-        window.activate(global.get_current_time());
-}
-
-function _openMenu(menu) {
-    menu.open();
-
-    let event = Clutter.get_current_event();
-    if (event && event.type() === Clutter.EventType.KEY_RELEASE)
-        menu.actor.navigate_focus(null, Gtk.DirectionType.TAB_FORWARD, false);
-}
-
-function _onMenuStateChanged(menu, isOpen) {
-    if (isOpen)
-        return;
-
-    let [x, y] = global.get_pointer();
-    let actor = global.stage.get_actor_at_pos(Clutter.PickMode.REACTIVE, x, y);
-    if (Me.stateObj.someWindowListContains(actor))
-        actor.sync_hover();
-}
-
+/**
+ * @param {Shell.App} app - an app
+ * @returns {number} - the smallest stable sequence of the app's windows
+ */
 function _getAppStableSequence(app) {
     let windows = app.get_windows().filter(w => !w.skip_taskbar);
     return windows.reduce((prev, cur) => {
@@ -58,7 +33,6 @@ function _getAppStableSequence(app) {
     }, Infinity);
 }
 
-
 class WindowContextMenu extends PopupMenu.PopupMenu {
     constructor(source, metaWindow) {
         super(source, 0.5, St.Side.BOTTOM);
@@ -282,10 +256,37 @@ const BaseButton = GObject.registerClass({
         return true;
     }
 
+    _openMenu(menu) {
+        menu.open();
+
+        let event = Clutter.get_current_event();
+        if (event && event.type() === Clutter.EventType.KEY_RELEASE)
+            menu.actor.navigate_focus(null, Gtk.DirectionType.TAB_FORWARD, false);
+    }
+
+    _minimizeOrActivateWindow(window) {
+        let focusWindow = global.display.focus_window;
+        if (focusWindow === window ||
+            focusWindow && focusWindow.get_transient_for() === window)
+            window.minimize();
+        else
+            window.activate(global.get_current_time());
+    }
+
+    _onMenuStateChanged(menu, isOpen) {
+        if (isOpen)
+            return;
+
+        let [x, y] = global.get_pointer();
+        let actor = global.stage.get_actor_at_pos(Clutter.PickMode.REACTIVE, x, y);
+        if (Me.stateObj.someWindowListContains(actor))
+            actor.sync_hover();
+    }
+
     _onPopupMenu(_actor) {
         if (!this._canOpenPopupMenu() || this._contextMenu.isOpen)
             return;
-        _openMenu(this._contextMenu);
+        this._openMenu(this._contextMenu);
     }
 
     _isFocused() {
@@ -362,7 +363,8 @@ class WindowButton extends BaseButton {
         this.label_actor = this._windowTitle.label_actor;
 
         this._contextMenu = new WindowContextMenu(this, this.metaWindow);
-        this._contextMenu.connect('open-state-changed', _onMenuStateChanged);
+        this._contextMenu.connect('open-state-changed',
+            this._onMenuStateChanged.bind(this));
         this._contextMenu.actor.hide();
         this._contextMenuManager.addMenu(this._contextMenu);
         Main.uiGroup.add_actor(this._contextMenu.actor);
@@ -382,9 +384,9 @@ class WindowButton extends BaseButton {
         }
 
         if (button === 1)
-            _minimizeOrActivateWindow(this.metaWindow);
+            this._minimizeOrActivateWindow(this.metaWindow);
         else
-            _openMenu(this._contextMenu);
+            this._openMenu(this._contextMenu);
     }
 
     _isFocused() {
@@ -518,14 +520,16 @@ class AppButton extends BaseButton {
 
         this._menuManager = new PopupMenu.PopupMenuManager(this);
         this._menu = new PopupMenu.PopupMenu(this, 0.5, St.Side.BOTTOM);
-        this._menu.connect('open-state-changed', _onMenuStateChanged);
+        this._menu.connect('open-state-changed',
+            this._onMenuStateChanged.bind(this));
         this._menu.actor.hide();
         this._menu.connect('activate', this._onMenuActivate.bind(this));
         this._menuManager.addMenu(this._menu);
         Main.uiGroup.add_actor(this._menu.actor);
 
         this._appContextMenu = new AppContextMenu(this);
-        this._appContextMenu.connect('open-state-changed', _onMenuStateChanged);
+        this._appContextMenu.connect('open-state-changed',
+            this._onMenuStateChanged.bind(this));
         this._appContextMenu.actor.hide();
         Main.uiGroup.add_actor(this._appContextMenu.actor);
 
@@ -592,7 +596,7 @@ class AppButton extends BaseButton {
                 this._singleWindowTitle.child = this._windowTitle;
                 this._windowContextMenu = new WindowContextMenu(this, this.metaWindow);
                 this._windowContextMenu.connect(
-                    'open-state-changed', _onMenuStateChanged);
+                    'open-state-changed', this._onMenuStateChanged.bind(this));
                 Main.uiGroup.add_actor(this._windowContextMenu.actor);
                 this._windowContextMenu.actor.hide();
                 this._contextMenuManager.addMenu(this._windowContextMenu);
@@ -631,7 +635,7 @@ class AppButton extends BaseButton {
             if (windows.length === 1) {
                 if (contextMenuWasOpen)
                     return;
-                _minimizeOrActivateWindow(windows[0]);
+                this._minimizeOrActivateWindow(windows[0]);
             } else {
                 this._menu.removeAll();
 
@@ -642,12 +646,12 @@ class AppButton extends BaseButton {
                     item._window = windows[i];
                     this._menu.addMenuItem(item);
                 }
-                _openMenu(this._menu);
+                this._openMenu(this._menu);
             }
         } else {
             if (contextMenuWasOpen)
                 return;
-            _openMenu(this._contextMenu);
+            this._openMenu(this._contextMenu);
         }
     }
 
@@ -774,13 +778,16 @@ class WindowList extends St.Widget {
             this._updateKeyboardAnchor();
         });
 
-        this._overviewHidingId = Main.overview.connect('hiding', () => {
+        this._overviewHidingId = Main.overview.connect('hidden', () => {
             this.visible = !Main.layoutManager.primaryMonitor.inFullscreen;
             this._updateKeyboardAnchor();
         });
 
         this._fullscreenChangedId =
             global.display.connect('in-fullscreen-changed', () => {
+                // Work-around for initial change from unknown to !fullscreen
+                if (Main.overview.visible)
+                    this.hide();
                 this._updateKeyboardAnchor();
             });
 
@@ -918,11 +925,8 @@ class WindowList extends St.Widget {
     }
 
     _updateKeyboardAnchor() {
-        if (!Main.keyboard.keyboardActor)
-            return;
-
-        let translationY = Main.overview.visible ? 0 : this.height;
-        Main.keyboard.keyboardActor.translation_y = -translationY;
+        const translationY = Main.overview.visible ? 0 : this.height;
+        Main.layoutManager.keyboardBox.translation_y = -translationY;
     }
 
     _onAppStateChanged(appSys, app) {
@@ -1170,6 +1174,9 @@ class Extension {
     }
 }
 
+/**
+ * @returns {Extension} - the extension's state object
+ */
 function init() {
     return new Extension();
 }
